import diffsync
from partial_date import PartialDate


class Residence(diffsync.DiffSyncModel):
    Label: str

    Country: str
    Finish: PartialDate | None
    Locality: str
    OLC: str
    PostalCode: str
    Region: str
    Start: PartialDate | None
    Street: str

    _modelname = "Residence"
    _identifiers = ("Label",)
    _attributes = (
        "Country",
        "Finish",
        "Locality",
        "OLC",
        "PostalCode",
        "Region",
        "Start",
        "Street",
    )

    @classmethod
    def create(
        cls,
        adapter: diffsync.Adapter,
        ids: dict,
        attrs: dict,
    ) -> "Residence":
        raise NotImplementedError

    def update(
        self,
        attrs: dict,
    ) -> "Residence":
        raise NotImplementedError

    def delete(
        self,
    ) -> "Residence":
        raise NotImplementedError



import click
import orjson
import tabulate
from dotenv import load_dotenv
from partial_date import PartialDate

from .__version__ import __version__
from .client import RDFClient

load_dotenv(
    override=True,
)

github_repo_owner_option = click.option(
    "--github-repo-owner",
    required=True,
    help="The owner of the repository.",
    envvar="RDF_GITHUB_REPO_OWNER",
)

github_repo_name_option = click.option(
    "--github-repo-name",
    required=True,
    help="The name of the repository.",
    envvar="RDF_GITHUB_REPO_NAME",
)

github_file_path_option = click.option(
    "--github-file-path",
    required=True,
    help="The path to the file in the repository.",
    envvar="RDF_GITHUB_FILE_PATH",
)

github_token_option = click.option(
    "--github-token",
    required=True,
    help="The GitHub personal access token.",
    envvar="RDF_GITHUB_TOKEN",
)

rdf_root = click.option(
    "--rdf-root",
    required=True,
    help="The root URL of the RDF object.",
    envvar="RDF_ROOT",
)

json_option = click.option(
    "--json",
    "output_format",
    flag_value="json",
    help="Output as JSON.",
)

output_format_option = click.option(
    "--table",
    "output_format",
    flag_value="table",
    default=True,
    help="Output as a table (default).",
)

iata_option = click.argument(
    "iata",
    nargs=-1,
    required=True,
    type=click.STRING,
)

version_option = click.version_option(
    version=__version__,
    prog_name="rdf-tools",
)


@click.group()
@click.pass_context
@github_file_path_option
@github_repo_name_option
@github_repo_owner_option
@github_token_option
@rdf_root
@version_option
def cli(
    ctx: click.Context,
    *args: list,
    **kwargs: dict,
) -> None:
    ctx.obj = RDFClient(
        *args,
        **kwargs,
    )


@cli.command()
@click.pass_context
@iata_option
@json_option
@output_format_option
def airlines(
    ctx: click.Context,
    output_format: str,
    *args: list,
    **kwargs: dict,
) -> None:
    output(
        *args,
        fn=ctx.obj.get_airlines,
        output_format=output_format,
        **kwargs,
    )


@cli.command()
@click.pass_context
@iata_option
@json_option
@output_format_option
def airports(
    ctx: click.Context,
    output_format: str,
    *args: list,
    **kwargs: dict,
) -> None:
    output(
        *args,
        fn=ctx.obj.get_airports,
        output_format=output_format,
        **kwargs,
    )


@cli.command()
@click.pass_context
@json_option
@output_format_option
def courses(
    ctx: click.Context,
    output_format: str,
    *args: list,
    **kwargs: dict,
) -> None:
    output(
        *args,
        fn=ctx.obj.get_courses,
        output_format=output_format,
        **kwargs,
    )


@cli.command()
@click.pass_context
@json_option
@output_format_option
def residences(
    ctx: click.Context,
    output_format: str,
    *args: list,
    **kwargs: dict,
) -> None:
    output(
        *args,
        fn=ctx.obj.get_residences,
        output_format=output_format,
        **kwargs,
    )


def output(
    fn: callable,
    output_format: str,
    *args: list,
    **kwargs: dict,
) -> None:
    objs = fn(
        *args,
        **kwargs,
    )

    if not objs:
        return

    # JSON output
    if output_format == "json":
        click.echo(
            orjson.dumps(
                [
                    {
                        **obj,
                        **{
                            k: str(v)
                            for k, v in obj.items()
                            if isinstance(v, PartialDate)
                        },
                    }
                    for obj in objs
                ],
                option=orjson.OPT_SORT_KEYS,
            ),
        )
        return

    # Table output
    if output_format == "table":
        click.echo(
            tabulate.tabulate(
                objs,
                headers="keys",
                tablefmt="pretty",
                colalign=["left" for _ in objs[0]],
            )
        )
        return

    raise NotImplementedError



import rdflib


class AirlineMixin:
    def get_airlines_query(
        self,
        iata: list[str] | None = None,
    ) -> str:
        return f"""
SELECT ?label ?iata ?icao WHERE {{
    VALUES (?value) {{ ( "{'" ) ( "'.join([i.upper() for i in iata])}" ) }}
    ?href ip:airlineIataCode ?value.
    ?href rdfs:label ?label .
    ?href ip:airlineIataCode ?iata .
    ?href ip:airlineIcaoCode ?icao .
    ?href a ic:airline .
}}
"""

    def get_airlines(
        self,
        *args: list,
        **kwargs: dict,
    ) -> rdflib.Graph:
        result = self.graph.query(
            self.get_airlines_query(
                *args,
                **kwargs,
            ),
        )

        return [
            {str(k): str(b[str(k)]) if str(k) in b else None for k in result.vars}
            for b in result.bindings
        ]



import diffsync


class Airport(diffsync.DiffSyncModel):
    iata: str

    label: str
    icao: str

    _modelname = "Airport"
    _identifiers = ("iata",)
    _attributes = (
        "label",
        "icao",
    )

    @classmethod
    def create(
        cls,
        adapter: diffsync.Adapter,
        ids: dict,
        attrs: dict,
    ) -> "Airport":
        raise NotImplementedError

    def update(
        self,
        attrs: dict,
    ) -> "Airport":
        raise NotImplementedError

    def delete(
        self,
    ) -> "Airport":
        raise NotImplementedError



from functools import cached_property
from typing import ClassVar

import diffsync

from illallangi.rdf import RDFClient
from illallangi.rdf.models import Residence


class ResidentialHistoryAdapter(diffsync.Adapter):
    def __init__(
        self,
        github_file_path: str,
        github_repo_name: str,
        github_repo_owner: str,
        github_token: str,
        rdf_root: str,
        *args: list,
        **kwargs: dict,
    ) -> None:
        super().__init__(*args, **kwargs)
        self.github_file_path = github_file_path
        self.github_repo_name = github_repo_name
        self.github_repo_owner = github_repo_owner
        self.github_token = github_token
        self.rdf_root = rdf_root

    Residence = Residence

    top_level: ClassVar = [
        "Residence",
    ]

    type = "rdf_residencehistory"

    @cached_property
    def client(self) -> RDFClient:
        return RDFClient(
            github_file_path=self.github_file_path,
            github_repo_name=self.github_repo_name,
            github_repo_owner=self.github_repo_owner,
            github_token=self.github_token,
            rdf_root=self.rdf_root,
        )

    def load(
        self,
    ) -> None:
        if self.count() > 0:
            return

        if self.partner.count() == 0:
            self.partner.load()

        for obj in self.client.get_residences():
            self.add(
                Residence(
                    label=obj["Label"],
                    start=obj["Start"],
                    finish=obj["Finish"],
                ),
            )



import rdflib
import requests

from illallangi.rdf.client.airline import AirlineMixin
from illallangi.rdf.client.airport import AirportMixin
from illallangi.rdf.client.course import CourseMixin
from illallangi.rdf.client.residence import ResidenceMixin


class RDFClient(
    AirlineMixin,
    AirportMixin,
    CourseMixin,
    ResidenceMixin,
):
    def __init__(
        self,
        github_file_path: str,
        github_repo_name: str,
        github_repo_owner: str,
        github_token: str,
        rdf_root: str,
    ) -> None:
        self.github_file_path = github_file_path
        self.github_repo_name = github_repo_name
        self.github_repo_owner = github_repo_owner
        self.github_token = github_token
        self.rdf_root = rdf_root

    @property
    def graph(self) -> rdflib.Graph:
        url = f"https://api.github.com/repos/{self.github_repo_owner}/{self.github_repo_name}/contents/{self.github_file_path}"
        headers = {
            "Authorization": f"token {self.github_token}",
            "Accept": "application/vnd.github.v3.raw",
        }

        response = requests.get(
            url,
            headers=headers,
            timeout=5,
        )

        response.raise_for_status()

        return rdflib.Graph().parse(data=response.content)



from typing import ClassVar

import diffsync

from illallangi.rdf import RDFClient
from illallangi.rdf.models import Airline, Airport


class AviationAdapter(diffsync.Adapter):
    Airline = Airline
    Airport = Airport

    top_level: ClassVar = [
        "Airline",
        "Airport",
    ]

    type = "rdf_aviation"

    def load(
        self,
        airline_iata: list[str] | None = None,
        airport_iata: list[str] | None = None,
    ) -> None:
        airline_iata = airline_iata or []
        airport_iata = airport_iata or []

        for obj in self.client.get_airlines(
            airline_iata,
        ):
            self.add(
                Airline(
                    iata=obj["iata"],
                    label=obj["label"],
                    icao=obj["icao"],
                ),
            )

        for obj in self.client.get_airports(
            airport_iata,
        ):
            self.add(
                Airport(
                    iata=obj["iata"],
                    label=obj["label"],
                    icao=obj["icao"],
                ),
            )

    def __init__(
        self,
        *args: list,
        **kwargs: dict,
    ) -> None:
        super().__init__()
        self.client = RDFClient(
            *args,
            **kwargs,
        )



import rdflib
from partial_date import PartialDate


class CourseMixin:
    def get_courses_query(
        self,
    ) -> str:
        return f"""
SELECT ?Start ?Finish ?Label ?Institution ?Street ?Locality ?Region ?PostalCode ?Country ?OLC WHERE {{
    <{ self.rdf_root }> ip:attendedCourse ?attendedCourse .
    OPTIONAL {{ ?attendedCourse ip:startTime ?Start }} .
    OPTIONAL {{ ?attendedCourse ip:endTime ?Finish }} .
    ?attendedCourse rdfs:label ?Label .
    ?attendedCourse ip:atInstitution ?atInstitution .

    OPTIONAL {{ ?atInstitution rdfs:label ?Institution . }}

    OPTIONAL {{ ?atInstitution v:Address ?address
        OPTIONAL {{ ?address v:street-address ?Street }}
        OPTIONAL {{ ?address v:locality ?Locality }}
        OPTIONAL {{ ?address v:region ?Region }}
        OPTIONAL {{ ?address v:postal-code ?PostalCode }}
        OPTIONAL {{ ?address v:country-name ?Country }}
    }}
    OPTIONAL {{ ?atInstitution ip:olc ?OLC }} .
}}
"""

    def get_courses(
        self,
        *args: list,
        **kwargs: dict,
    ) -> rdflib.Graph:
        result = self.graph.query(
            self.get_courses_query(
                *args,
                **kwargs,
            ),
        )

        return sorted(
            [
                {
                    **{
                        str(k): b[str(k)].value if str(k) in b else None
                        for k in result.vars
                    },
                    "Start": PartialDate(b["Start"].value)
                    if "Start" in b and b["Start"].value not in ["Unknown"]
                    else None,
                    "Finish": PartialDate(b["Finish"].value)
                    if "Finish" in b and b["Finish"].value not in ["Unknown"]
                    else None,
                }
                for b in result.bindings
            ],
            key=lambda x: str(x["Start"]),
        )
